@@ -1,5 +1,5 @@
 /*
- * (c) 2014 by van Hauser / THC
+ * (c) 2015 by van Hauser / THC
  *
  * THC IPv6 Attack Library Header Files
  *
@@ -17,10 +17,14 @@
   #include <openssl/rsa.h>
 #endif
 
-#define VERSION 	"v2.7"
+#define VERSION 	"v3.0"
 #define AUTHOR 		"van Hauser / THC <vh@thc.org>"
 #define RESOURCE	"www.thc.org"
 
+#define THC_SPLITCONNECT_PORT 64446
+#define THC_SPLITCONNECT_FROM_BYTE 0xff
+#define THC_SPLITCONNECT_TO_BYTE   0xee
+
 // you can change the following if you want to:
 #define SHOW_LIBRARY_ERRORS 1
 
